#include <iostream> // For writing to the standard output.
#include <fstream> // For reading input files.

#include "graph.hpp"
#include "../blossom5-v2.05.src/PerfectMatching.h"

int main(int argc, char** argv)
{
   // If you don't want to write CP:: all the time,
   // using a namespace in the implementation of some function is perfectly ok.
   using namespace CP;

   if (argc != 3)
   {
      std::cout << "Expected exactly two arguments (path to input graph and path to output graph)!" << std::endl;
      return EXIT_FAILURE; // return 1 would do the same, but is way too easy to mix up!
   }

   std::fstream input_file_graph{argv[1]};
   Graph const graph = Graph::read_dimacs(input_file_graph);
   if (graph.num_edges() == graph.num_nodes() * (graph.num_nodes() - 1))
   {
      std::cout << "Expected input graph to be complete graph!" << std::endl;
      std::cout << "Your algorithm should not do this but since this algorithm" 
                << " only computes a perfect matching on the input, it wants to have a "
                << "complete graph. Change this when using this routine for your purposes."
                << std::endl;
      return EXIT_FAILURE;
   }
   else if (graph.num_nodes() % 2 != 0)
   {
      std::cout << "Expected input graph to have even number of vertices! This is also " 
                << " not expected in your routine but we need this for the existance of "
                << " a perfect matching." << std::endl;
      return EXIT_FAILURE;
   }
   else
   {
      PerfectMatching solver{static_cast<int>(graph.num_nodes()), static_cast<int>(graph.num_edges())};
      for (EdgeId edge_id{0}; edge_id < graph.num_edges(); ++edge_id)
      {
         solver.AddEdge(
	        graph.halfedge(2*edge_id + 1).target(),
	        graph.halfedge(2*edge_id + 0).target(),
	        graph.edge_weight(edge_id));
      }
 
      solver.Solve();

      Graph solution{graph.num_nodes()};
      for (EdgeId edge_id{0}; edge_id < graph.num_edges(); ++edge_id)
      {
         if (solver.GetSolution(edge_id))
	     {
            solution.add_edge(
	           graph.halfedge(2*edge_id + 1).target(),
	           graph.halfedge(2*edge_id + 0).target(),
	           graph.edge_weight(edge_id));
	     }
      }
      std::ofstream output_file{argv[2]};
      output_file << solution;
      output_file.close();

      return EXIT_SUCCESS; 
   }

}

